{% if theme_logo %}
<p class="logo">
  <a href="{{ pathto(master_doc) }}">
    <img class="logo" src="{{ pathto('_static/' ~ theme_logo, 1) }}" alt="Logo"/>
    {% if theme_logo_name|lower == 'true' %}
    <h1 class="logo logo-name">{{ project }}</h1>
    {% endif %}
  </a>
</p>
{% else %}
<h1 class="logo"><a href="{{ pathto(master_doc) }}">{{ project }} <small>v{{ version }}</small></a></h1>
{% endif %}

{% if theme_description %}
<p class="blurb">{{ theme_description }}</p>
{% endif %}

{% set githubpath = theme_github_user + '/' + theme_github_repo %}

<a class="badge" href="">
    <img
        alt="Project License"
        src="https://img.shields.io/pypi/l/{{ project }}.svg"
    />
</a>

<a class="badge" href="https://pypi.org/project/{{ project }}">
    <img
        alt="PyPI Package Version"
        src="https://img.shields.io/pypi/v/{{ project }}.svg"
    />
</a>

<a class="badge" href="https://pypi.org/project/{{ project }}">
    <img
        alt="Supported Python Versions"
        src="https://img.shields.io/pypi/pyversions/{{ project }}.svg"
    />
</a>

<a class="badge" href="https://travis-ci.org/{{ githubpath }}">
    <img
        alt="Travis CI Build Status"
        src="https://img.shields.io/travis/{{ githubpath }}.svg?logo=travis"
    />
</a>

<a class="badge" href="https://coveralls.io/github/{{ githubpath }}">
    <img
        alt="Coveralls Code Coverage"
        src="https://img.shields.io/coveralls/github/{{ githubpath }}.svg?logo=coveralls"
    />
</a>

<a class="badge" href="https://github.com/{{ githubpath }}">
    <img
        alt="GitHub watchers"
        src="https://img.shields.io/github/watchers/{{ githubpath }}.svg?label=watch&style=social"
    />
</a>

